#include <ESP8266WiFi.h>
#include <stdlib_noniso.h>

//....................... Параметры WIFI ............................
const char* ssid = "xxxxxxxxxxxxxx";        // Введите имя WIFI сети
const char* password = "xxxxxxxx";          // Введите пароль WIFI сети

//...................... Параметры сервиса ThingsPeak ...............
const char* host = "api.thingspeak.com";
const char* APIkey   = "xxxxxxxxxxxxxxxx";  // Введите API key thingspeak

//...................... Датчик DHT22 (DHT11) .......................
#include <DHT.h>
#define DHTPIN 12                         // Порт подключения датчика DHT22 или DHT11,
#define DHTTYPE DHT22                     // Тип датчика: DHT22 или DHT11
DHT dht(DHTPIN, DHTTYPE);

//...................... Датчик DS18B20 .............................
#include <OneWire.h>
#include <DallasTemperature.h>
#define ONE_WIRE_BUS 14                   // Порт подключения датчика DS18B20
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature DS18B20(&oneWire);

//....................... Датчик BMP .................................
#include <Adafruit_BMP085.h>
#define I2C_SCL 2                     // SCL порт для BMP085/BMP180
#define I2C_SDA 0                     // SDA порт для BMP085/BMP180
Adafruit_BMP085 bmp;
bool bmp085_present=true;

//.....................................................................
float oldTemp;
float dst,bt,bp,h,t;
char dstmp[20],btmp[20],bprs[20];
//===============================================================
void setup() 
{
  Serial.begin(115200);
  delay(10);
  Serial.println();
//................Подключение к WIFI сети......................
  WiFi.mode(WIFI_STA);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) 
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

 //....................... BMP..................................
  Wire.begin(I2C_SDA, I2C_SCL);
  delay(10);
  if (!bmp.begin()) 
  {
    Serial.println("No BMP sensor detected!");
    bmp085_present=false;
  }
 //.......................DHT...................................
   dht.begin();
}
//================================================================
void loop() 
{
  //.....................DS18b20...................................
  DS18B20.requestTemperatures();
  dst = DS18B20.getTempCByIndex(0);
  Serial.print("Temperature ds: ");
  Serial.println(dst);
  dtostrf(dst, 8, 2, dstmp);

  //.........................BMP..................................
  if(bmp085_present) 
  {
     bt = bmp.readTemperature();               //(bmp.readTemperature() * 9/5) + 32;
     Serial.print("Temperature bmp: ");
     Serial.println(bt);
     dtostrf(bt, 8, 2, btmp);
     bp = bmp.readPressure()/133.3224;         // В мм. ртутного столба
     Serial.print("Pressure bmp: ");
     Serial.println(bp);
     dtostrf(bp, 8, 2, bprs);
  }

//.........................DHT.......................................
  h = dht.readHumidity();
  t = dht.readTemperature();
  if (isnan(h) || isnan(t)) 
  {
    Serial.println("Failed to read from DHT sensor!");
    return;
  }
  Serial.print("Temperature dht: ");
  Serial.println(t);
  Serial.print("Humidity dht: ");
  Serial.println(h);
  //..................................................................
  Serial.println("");
  Serial.print("Connecting to ");
  Serial.println(host);
  WiFiClient client;
  
  const int httpPort = 80;
  if (!client.connect(host, httpPort)) 
  {
    Serial.println("connection failed");
    return;
  }
 
//......................................................................
  String url = "/update?key=";
         url += APIkey;
         url += "&field1=";
         url += t;                     //DHT Temp
         url += "&field2=";
         url += h;                     //DHT Humidity  
         url += "&field3=";
         url += dstmp;                 //DS Temp;
         url += "&field4=";
         url += btmp;                  //BMP Temp;
         url += "&field5=";
         url += bprs;                  //BMP Press;
         url += "\r\n\r\n";
 
  Serial.println("Sending data: ");
  Serial.println(url);
                                                // This will send the request to the server
  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + host + "\r\n" +
               "Connection: close\r\n\r\n");
  delay(10);
                                                // Read all the lines of the reply from server and print them to Serial
  while (client.available()) 
  {
    String line = client.readStringUntil('\r');
    Serial.print(line);
  }
  Serial.println();
  Serial.println("closing connection");

  
  Serial.println("Waiting");
  for(unsigned int i = 0; i < 20; i++)   // thingspeak необходимо задержка между обновлениями минимум 15 сек.
  {
    delay(3000);                         
  }
}
